#pragma once
#include <unordered_map>
#include <type_traits>

#include "account_t.h"
#include <us/gov/types.h>
#include <us/gov/io/seriable_map.h>

namespace us::gov::cash {

    struct accounts_t final: io::seriable_map<hash_t, account_t> {
        using b = io::seriable_map<hash_t, account_t>;

        static const char* KO_69213;

    public:
        accounts_t();
        ~accounts_t();

    public:
        using batch_item = account_t;
        struct batch_t final: unordered_map<hash_t, batch_item> {
            void add(const hash_t& address, const batch_item&);
        };

    public:
        void add(const batch_t& batch);
        account_t merge() const;

    public:
        cash_t get_balance() const;
        bool pay(const hash_t& k, const cash_t& amount);
        bool withdraw(const hash_t& k, const cash_t& amount);
        cash_t sum() const;

    public:
        ko print_data(const hash_t& addr, ostream&) const;
        void print_kv_b64(const hash_t& addr, ostream&) const;
        void dump(const string& prefix, int detail, ostream&) const;

    public:
        void list_files(const hash_t& addr, const string& path, ostream&) const;
        void get_files(vector<pair<hash_t, pair<hash_t, fileattr_t>>>&) const;
        void get_files(vector<pair<hash_t, fileattr_t>>&) const;
        void get_files(vector<pair<hash_t, uint32_t>>&) const;
        void get_files(set<hash_t>&) const;
        hash_t file_hash(const hash_t& addr, const string& path) const;

    public:
        void hash_data_to_sign(sigmsg_hasher_t&) const;
        void hash_data(hasher_t&) const;

    };

    static_assert(std::is_convertible<accounts_t*, io::blob_writer_t::writable*>::value, "KO 73822");

}

/*
namespace us::gov::io {
    template<> inline ko blob_reader_t::readD(const us::gov::socket::datagram& d, us::gov::cash::accounts_t& o) { return o.read(d); }
    template<> inline datagram* blob_writer_t::get_datagram(channel_t channel, svc_t svc, seq_t seq, const us::gov::cash::accounts_t& o) { return o.get_datagram(channel, svc, seq); }
}
*/

