#pragma once
#include <map>
#include <iostream>
#include <inttypes.h>

#include <us/gov/types.h>
#include <us/gov/io/seriable.h>
#include <us/gov/crypto/ec.h>

#include "app.h"
#include "signed_data.h"
#include "types.h"

namespace us::gov::engine {

    struct local_deltas_t final: map<appid_t, app::local_delta*>, signed_data, virtual io::seriable {
        using b = map<appid_t, app::local_delta*>;
        using s = io::seriable;

        local_deltas_t(): id(0) {}
        local_deltas_t(const local_deltas_t&) = delete;
        ~local_deltas_t();

    public:
        void clear();

        using s::get_datagram;
        datagram* get_datagram(channel_t, seq_t) const;

    public:
        void hash_data_to_sign(sigmsg_hasher_t&) const override;
        void hash_data(hasher_t&) const override;
        void dumpX(ostream&) const;

    public:
        static constexpr serid_t serid{'L'};
        serid_t serial_id() const override { return serid; }
        size_t blob_size() const override;
        void to_blob(blob_writer_t&) const override;
        ko from_blob(blob_reader_t&) override;

    public:
        ts_t id;
    };

}
/*
namespace us::gov::io {
    template<> inline ko blob_reader_t::readD(const us::gov::socket::datagram& d, us::gov::engine::local_deltas_t& o) { return o.read(d); }
    template<> inline datagram* blob_writer_t::get_datagram(channel_t channel, svc_t svc, seq_t seq, const us::gov::engine::local_deltas_t& o) { return o.get_datagram(channel, svc, seq); }
}
*/
