#pragma once
#include <unordered_map>
#include <us/gov/likely.h>

namespace us::gov::engine {

    template<typename d>
    struct average_merger {
        void merge(d& v1, const d& v2) const {
            v1 += v2;
        }
        void end_merge(d& v, const uint64_t& multiplicity) const {
            double m = multiplicity;
            v /= m;
        }
    };

    template<typename d>
    struct majority_merger {
        void merge(const d& v) {
            auto i = m.find(v);
            if (i != m.end()) {
                ++i->second;
                return;
            }
            m.emplace(v, 1);
        }

        void end_merge(d& v) {
            const d* x = nullptr;
            uint64_t z = 0;
            for (auto& i: m) {
                if (i.second > z) {
                    z = i.second;
                    x = &i.first;
                }
            }
            if (likely(z != 0)) {
                v = *x;
            }
            else {
                v = 0;
            }
            m.clear();
        }
        unordered_map<d, uint64_t> m;
    };

}

