#pragma once
#include <us/gov/types.h>
#include <us/gov/io/seriable.h>

namespace us::gov::peer {

    struct account_t: virtual io::seriable {

        account_t() {}
        account_t(host_t net_address, port_t port, uint16_t seen, uint16_t w);
        account_t(const account_t&);

        account_t& operator = (int zero);
        bool operator == (const account_t&) const;
        account_t& operator = (const account_t&);

        void dump(ostream&) const;
        void hash_data_to_sign(sigmsg_hasher_t&) const;
        void hash_data(hasher_t&) const;
        void dump_as_seeds(ostream&) const;
        ko check(channel_t) const;
        string endpoint() const;
        void print_endpoint(ostream&) const;
        void hash(crypto::ripemd160&) const;

    public:
        size_t blob_size() const override;
        void to_blob(blob_writer_t&) const override;
        ko from_blob(blob_reader_t&) override;

    public:
        host_t net_address;
        port_t port;
        uint16_t seen;
        uint16_t w;
    };

}

namespace std {
    template <>
    struct hash<us::gov::peer::account_t> {
        size_t operator()(const us::gov::peer::account_t&) const noexcept;
    };
}

