#pragma once
#include <mutex>

#include <us/gov/types.h>
#include <us/gov/io/seriable.h>
#include <us/gov/peer/account_t.h>
#include <us/gov/peer/nodes_t.h>
#include <us/gov/types.h>

namespace us::gov::traders {

    struct db_t final: io::seriable {
        using account_t = peer::account_t;
        using wallets_t = peer::nodes_t;

        db_t();
        void hash(hasher_t&) const;
        void dump(const string& prefix, ostream&) const;
        void clear();
        void collect(vector<tuple<hash_t, host_t, port_t>>&) const;
        void collect(vector<hash_t>&) const;
        pair<ko, account_t> lookup(const hash_t&) const;
        pair<ko, tuple<hash_t, host_t, port_t>> random_wallet() const;

    public:
        size_t blob_size() const override;
        void to_blob(blob_writer_t&) const override;
        ko from_blob(blob_reader_t&) override;

    public:
        wallets_t wallets;
        mutable mutex mx_wallets;
    };

}

