#pragma once
#include <string>

#include <us/gov/config.h>
#include <us/gov/io/shell_args.h>

#include "types.h"

namespace us::wallet::cli {

    struct params {
        using shell_args = us::gov::io::shell_args;
        params();
        params(const shell_args&);

        void constructor();

        enum output_mode {
            om_human,
            om_text,
            om_xml,
            om_json,

            num_modes
        };

        constexpr static array<const char*, num_modes> omstr = {"human", "text", "xml", "json"};

        string get_output_mode() const { return omstr[om]; }
        string get_home_wallet() const;
        string get_home_wallet_rpc_client() const;
        string get_home_gov() const;
        string get_home() const;
        void dump(ostream&) const;
        void dump(const string& prefix, ostream&) const;
        void connecting_to(ostream&) const;

    public:
        channel_t channel{CFG_CHANNEL};
        string homedir;
        bool daemon{false};
        bool advanced{true};
        bool ncurses{false};
        bool verbose{false};
        uint8_t max_trade_edges{8};
        uint8_t max_devices{4};
        uint8_t workers;
        string subhome;
        string cmd;
        shell_args args;
        port_t listening_port{CFG_WALLET_PORT};
        port_t published_port{CFG_WALLET_PPORT};
        pin_t pin{0};
        bool local{false}; //default RPC mode
        string walletd_host{"localhost"};
        port_t walletd_port{CFG_WALLET_PORT};
        string backend_host{"localhost"};
        port_t backend_port{CFG_PORT};
        bool rpc__connect_for_recv{true};
        bool rpc__stop_on_disconnection{true};
        output_mode om{om_human};
        bool names{true};
        string downloads_dir;
        #if CFG_FCGI == 1
            bool fcgi{false};
        #endif
        bool banner{true};
        string logd{LOGDIR};
        bool nolog{false};
        uint16_t rpc_timeout_secs{CFG_DGRAM_ROUNDTRIP_TIMEOUT_SECS};
    };

}

