#pragma once
#include <chrono>
#include <utility>
#include <map>

#include <us/gov/config.h>
#include <us/gov/cli/rpc_daemon_t.h>
#include <us/gov/relay/pushman.h>
#include <us/gov/engine/daemon_t.h>
#include <us/gov/engine/track_status_t.h>

#include <us/wallet/trader/trades_t.h>
//#include <us/wallet/trader/bootstrap/protocols_t.h>

#include "users_t.h"
#include "data_sources_t.h"
#include "devices_t.h"
#include "types.h"
#include "svcfish_t.h"
#include "bookmark_index_t.h"

namespace us::wallet::wallet {
    struct local_api;
}

namespace us::wallet::engine {

    struct peer_t;
    struct daemon_t;

    struct mezzanine: bgtask {
        using b = bgtask;
        mezzanine(daemon_t*);
        daemon_t* d;
    };

    /// Main algorithm of the wallet daemon, being a relay daemon (access to remote wallets and devices) and a housekeeping thread (listed in the public wallet directory))
    struct daemon_t final: us::gov::relay::daemon_t, us::wallet::engine::mezzanine {
        using b = us::gov::relay::daemon_t;
        using housekeeping = us::wallet::engine::mezzanine;
        using dispatcher_t = us::gov::datagram::dispatcher_t;
        using pushman = us::gov::relay::pushman;
        using track_status_t = us::gov::engine::daemon_t::ev_track_t::status_t;
        using trades_t = us::wallet::trader::trades_t;

        static const char* KO_20193, *KO_20197;

    public:
        daemon_t(channel_t, const keys_t&, port_t, pport_t, const string& home, const shostport_t& gov, uint8_t max_trade_edges, uint8_t max_devices, uint8_t workers, const string& downloads_dir);
        virtual ~daemon_t() override;

    public:
        using protocols_t = us::wallet::trader::bootstrap::protocols_t;

        socket::client* create_client(sock_t sock) override;
        const keys_t& get_keys() const override { return id; }
        ko authorize_device(const pub_t&, pin_t pin, request_data_t&);
        bool send_response(socket::peer_t*, datagram*, const string& payload);
        bool send_response(socket::peer_t*, datagram*, const vector<uint8_t>& payload);
        bool send_error_response(socket::peer_t*, datagram*, const string& error);
        void on_destroy_(socket::client&) override;
        ko start();
        ko wait_ready(const time_point& deadline) const;
        ko wait_ready(int seconds_deadline) const;
        void stop();
        void join();
        bool is_active() const;
        inline bool isup() const { return b::isup(); }
        pport_t get_pport() const { return b::pport; }
        void on_peer_wallet(const hash_t& addr, host_t address, pport_t rpport);
        ko announce_wallet();
        ko register_w_(host_t address);
        void wait(const chrono::seconds&);
        void run();
        void onwakeup();
        ko wait_auth();
        void configure_gov_rpc_daemon();
        string wallet_home(const string& subhome) const;
        string wallet_template_home(const string& templatename) const;
        bool has_home(const string& subhome) const;
        void on_tx_tracking_status(const track_status_t&);
        void hook_new_wallet(const string& subhome, const string& wallet_template);
        void hook_new_wallet__worker(const string& subhome, const string& wallet_template);

        ko lookup_wallet(const hash_t& addr, hostport_t&);
        #ifdef CFG_TOPOLOGY_RING
            inline string homedir() const override { return home; }
        #endif

        string rewrite(ko) const override;
        ko handle_conf(const string& key, const string& value, string& ans);

    public:
        virtual void upgrade_software();

    public:
        using gov_rpc_daemon_t = gov::cli::rpc_daemon_t;
        using gov_dto = gov_rpc_daemon_t::peer_type;

    public:
        static svcfish_t svcfish;

        keys id;
        string downloads_dir;
        condition_variable cv;
        mutex mx;
        hash_t trusted_address; /// automatic updates
        string home;
        bookmark_index2_t bookmark_index;
        devices_t devices;
        trades_t trades;
        users_t users;
        data_sources_t data_sources;
        pushman pm;
        gov_rpc_daemon_t gov_rpc_daemon;

    };

}

