#pragma once
#include <us/gov/io/seriable.h>
#include <us/gov/io/seriable_vector.h>
#include <us/gov/socket/types.h>
#include <us/gov/config.h>

#include "ip4_endpoint_t.h"

namespace us::wallet::engine {

    struct wallet_connection_t final_: gov::io::seriable {

    public:
        wallet_connection_t();
        wallet_connection_t(const string& nm, const string& subhome, const ip4_endpoint_t& ep);
        wallet_connection_t(uint64_t ts_, const string& addr_, const string& subhome, const string& nm, const string& ssid_, const ip4_endpoint_t&);
        wallet_connection_t(const wallet_connection_t& other);

    public:
        wallet_connection_t copy() const;
        string log_string() const;
        ko set_endpoint(const ip4_endpoint_t&);
        string get_title() const;
        void dump(const string& pfx, ostream&) const;

    public: //serialization blob
        size_t blob_size() const override;
        void to_blob(blob_writer_t&) const override;
        ko from_blob(blob_reader_t&) override;

    public:
        string name_;
        string ssid;
        string addr;
        string subhome;
        ip4_endpoint_t ip4_endpoint;
        uint64_t ts{0};
    };

    struct wallet_connections_t: gov::io::seriable_vector<wallet_connection_t> {
        void dump(ostream&) const;
    };

}

