#pragma once
#include <string>

#include <us/wallet/trader/kv.h>
#include <us/wallet/trader/protocol_selection_t.h>

#include "handshake_t.h"
#include "a_t.h"

namespace us::wallet::trader::bootstrap {

    using namespace std;

    /// Handshake modes:
    /// *---a---->*---b----> endpoint w/o protocol_selection, then protocol_selection
    /// *--------c---------> endpoint with protocol_selection

    struct b1_t: handshake_t {
        using params_t = trader::kv;

        b1_t();
        b1_t(const protocol_selection_t&, const params_t&);

    public: //serialization
        size_t blob_size() const override;
        void to_blob(blob_writer_t&) const override;
        ko from_blob(blob_reader_t&) override;

    public:
        protocol_selection_t protocol_selection;
        params_t params;
    };

    struct b2_t: handshake_t {
        using params_t = trader::kv;
        using personality_proof_t = trader::personality::proof_t;

        b2_t();
        b2_t(personality_proof_t&&);
        b2_t(personality_proof_t&&, params_t&&);

    public: //serialization
        size_t blob_size() const override;
        void to_blob(blob_writer_t&) const override;
        ko from_blob(blob_reader_t&) override;

    public:
        params_t params;
        personality_proof_t personality_proof;
    };

    struct b3_t: handshake_t {
        using params_t = trader::kv;

        b3_t();
        b3_t(params_t&&);

    public: //serialization
        size_t blob_size() const override;
        void to_blob(blob_writer_t&) const override;
        ko from_blob(blob_reader_t&) override;

    public:
        params_t params;
    };

}

