package plebble.trader;
import java.util.ArrayDeque;                                                                   // ArrayDeque
import java.util.concurrent.locks.Condition;                                                   // Condition
import us.gov.socket.datagram;                                                                 // datagram
import java.util.HashMap;                                                                      // HashMap
import static us.gov.crypto.ripemd160.hash_t;                                                  // hash_t
import static us.stdint.*;                                                                     // *
import static us.tuple.*;                                                                      // *
import static us.ko.is_ko;                                                                     // is_ko
import us.ko;                                                                                  // ko
import static us.ko.ok;                                                                        // ok
import us.pair;                                                                                // pair
import java.util.Queue;                                                                        // Queue
import java.util.concurrent.locks.ReentrantLock;                                               // ReentrantLock

public class datagram_dispatcher_t implements datagram.dispatcher_t, Runnable {


    public static interface handler_t {
        void on_push(final hash_t target_tid, final uint16_t code, final byte[] payload);
    }

    public datagram_dispatcher_t(int num_workers) {
        workers = new Thread[num_workers];
        for (int i = 0; i < workers.length; ++i) {
            workers[i] = new Thread(this);
            workers[i].start();
        }
    }

    //2. push datagram picked from q by a worker thread process the datagram
    @Override public void run() {
        lock.lock();
        try {
            while(true) {
                datagram d = q.poll();
                if (d != null) {
                    lock.unlock();
                    process(d);
                    lock.lock();
                }
                else {
                    cv.await();
                }
            }
        }
        catch (InterruptedException e) {
        }
        finally {
            lock.unlock();
        }
    }

    public int connect_sink(handler_t s) {
        mx.lock();
        int id = nextid++;
        try {
            sinks.put(id,s);
        }
        finally {
            mx.unlock();
        }
        return id;
    }

    public void disconnect_sink(int id) {
        mx.lock();
        try {
            sinks.remove(id);
        }
        finally {
            mx.unlock();
        }
    }

    //2. push datagram enters q. Called by hmi guts
    @Override public boolean dispatch(datagram d) {
        lock.lock();
        try {
            q.add(d);
            cv.signal();
        }
        finally {
            lock.unlock();
        }
        return true;
    }

    //4. on push is called on every subscriptor (sinks)
    public void propagate(hash_t tid, uint16_t code, byte[] payload) {
        //System.out.println("propagate: " + tid.encode() + " " + code.value + " sz " + payload.length);
        mx.lock();
        try {
            for (handler_t s: sinks.values()) {
                if (s == null) {
                    continue;
                }
                s.on_push(tid, code, payload);
            }
        }
        finally {
            mx.unlock();
        }
    }

    //3. Decodes the datagram into trade id, code m payload, and propagates it
    public void process(datagram d) {
        try {
            switch(d.service.value) {
                case us.gov.protocol.relay_push: {
                    us.wallet.cli.rpc_peer_t.push_in_dst_t o_in = new us.wallet.cli.rpc_peer_t.push_in_dst_t();
                    us.gov.io.blob_reader_t reader = new us.gov.io.blob_reader_t(d);
                    ko r = reader.read(o_in);
                    if (is_ko(r)) {
                        return;
                    }
                    propagate(o_in.tid, o_in.code, o_in.blob.value);
                    return;
                }
                default:
                    return;
                }
        }
        catch(Exception e) {
        }
    }

    HashMap<Integer, handler_t> sinks = new HashMap<Integer, handler_t>();

    Thread[] workers;
    int nextid = 0;
    Queue<datagram> q = new ArrayDeque<datagram>();
    ReentrantLock lock = new ReentrantLock();
    Condition cv = lock.newCondition();
    ReentrantLock mx = new ReentrantLock();
}

