#pragma once
#include <array>

#include <us/gov/id/peer_t.h>
#include <us/gov/config.h>
#include <us/gov/types.h>

#include "api.h"
#include "types.h"

namespace us::gov::auth {

    struct peer_t: base_ns::peer_t, api {
        using b = base_ns::peer_t;
        using request_data_t = us::gov::id::request_data_t;

        enum stage_t {
            denied = 0,
            authorized,
            num_stages
        };

        constexpr static array<const char*, num_stages> stagestr = {"denied", "authorized"};

        using b::peer_t;

    public:
        bool process_work(datagram*) override;

    public:
        ko verification_completed(pport_t, pin_t, request_data_t&) override;
        virtual ko authorize(const pub_t&, pin_t, request_data_t&) = 0;

    public:
        void dump_all(const string& prefix, ostream&) const override;
        void dump(const string& prefix, ostream&) const;

    public:
    #ifdef has_us_gov_auth_api
        #include <us/api/generated/gov/c++/auth/hdlr_override>
        #include <us/api/generated/gov/c++/auth/hdlr_svc_handler-hdr>
        #include <us/api/generated/gov/c++/auth/cllr_override>
    #endif

    public:
    #if CFG_COUNTERS == 1
        struct counters_t {
           uint32_t successful_authorizations{0};
           uint32_t failed_authorizations{0};

           void dump(ostream&) const;
        };
        static counters_t counters;
    #endif

    public:
        stage_t stage{denied};
    };

}

