#include <us/gov/crypto/ripemd160.h>
#include <us/gov/engine/protocol.h>

#include "hmi.h"
#include "types.h"
#include "rpc_daemon_t.h"

#define loglevel "gov/cli"
#define logclass "rpc_daemon_t"
#include <us/gov/logs.inc>

using namespace us::gov::cli;
using c = us::gov::cli::rpc_daemon_t;

c::rpc_daemon_t(channel_t channel, const keys_t& keys, const shostport_t& shostport, role_t role, dispatcher_t* dispatcher): b(channel, dispatcher), id(keys), shostport(shostport), role(role), parent(nullptr) {
    log("constructor");
}

c::rpc_daemon_t(hmi& parent, const keys_t& keys, const shostport_t& shostport, role_t role, dispatcher_t* dispatcher): b(parent.p.channel, dispatcher), id(keys), shostport(shostport), role(role), parent(&parent) {
    log("constructor-hmi");
}

socket::client* c::create_client() {
    log("create_client");
    return new rpc_peer_t(*this);
}

ko c::connect() { //callback. called from the depths
    return connect(0, request_data_t());
}

ko c::connect(pin_t pin, request_data_t&& request_data) {
    log("connect with pin", pin);
    auto r = get_peer().connect(shostport, 0, pin, role, move(request_data), true);
    if (is_ko(r)) {
        return r;
    }
    peer->disable_recv_timeout();
    cv_connected.notify_all();
    return ok;
}

void c::on_connect(socket::client& peer, ko r) {
    log("on_connect", (is_ko(r) ? r : "ok"));
    b::on_connect(peer, r);
    if (parent) parent->on_connect(r);
}

void c::verification_completed(bool verif_ok) {
    log("verification_completed ", verif_ok);
    if (parent) parent->verification_completed(verif_ok);
}

void c::on_I_disconnected(const string& reason) {
    log("on_I_disconnected ", reason);
    if (parent) parent->on_I_disconnected(reason);
}

void c::on_peer_disconnected(const string& reason) {
    log("peer disconnected. Reason: ", reason);
    if (parent) parent->on_peer_disconnected(reason);
}

void c::verification_result(request_data_t&& request_data) {
    log("verification_result", request_data);
    if (parent) parent->verification_result(move(request_data));
}

void c::apihelp(const string& prefix, ostream& os) {
    os << prefix << "track <xtime>           Tracking info for transaction.\n";
    os << prefix << "n|nodes                 Node list.\n";
    os << prefix << "lookup_node <addr>      Node info by addresas.\n";
    os << prefix << "w|wallets               Wallet list.\n";
    os << prefix << "lookup_wallet <addr>    Wallet info by address.\n";
    os << prefix << "fetch_accounts {<addr>} Accounts by addreses.\n";
}

void c::on_stop() {
    log("on_stop");
    b::on_stop();
    if (parent) parent->on_stop();
}

void c::upgrade_software() {
    log("Peer is signaling the existence of a upgrade_software.");
    if (parent) parent->upgrade_software();
}

