#include <us/gov/io/cfg0.h>
#include <us/gov/io/blob_reader_t.h>
#include <us/gov/io/blob_writer_t.h>

#include "fileattr_t.h"

#define loglevel "gov/dfs"
#define logclass "fileattr_t"
#include "logs.inc"

using namespace us::gov::dfs;
using c = us::gov::dfs::fileattr_t;
using us::ko;

c::fileattr_t(): sz(0) {
}

c::fileattr_t(const string& path, uint32_t sz): path(us::gov::io::cfg0::rewrite_path(path)), sz(sz) {
}

void c::hash_data_to_sign(crypto::ec::sigmsg_hasher_t& h) const {
    h.write(path);
    h.write(sz);
}

void c::hash_data(crypto::ripemd160& h) const {
    h.write(path);
    h.write(sz);
}

void c::dump(ostream& os) const {
    os << path << ' ' << sz << " bytes\n";
}

size_t c::blob_size() const {
    return blob_writer_t::blob_size(sz) + blob_writer_t::blob_size(path);
 }

void c::to_blob(blob_writer_t& writer) const {
    writer.write(sz);
    writer.write(path);
}

ko c::from_blob(blob_reader_t& reader) {
    {
        auto r = reader.read(sz);
        if (is_ko(r)) return r;
    }
    {
        auto r = reader.read(path);
        if (is_ko(r)) return r;
    }
    return ok;
}

