#include <us/gov/cash/accounts_t.h>

#include "local_deltas_t.h"
#include "api.h"
#include "types.h"
#include "svcfish_t.h"
#include "rpc_daemon_t.h"
#include "net_daemon_t.h"
#include "daemon_t.h"
#include "track_status_t.h"
#include "rpc_peer_t.h"

#define loglevel "gov/engine"
#define logclass "rpc_peer_t"
#include "logs.inc"
#include <us/gov/socket/dto.inc>

namespace {
    using t = us::gov::engine::local_deltas_t;
    template<> datagram* write_datagram(channel_t channel, svc_t svc, seq_t seq, const t& o) {
        return o.get_datagram(channel, svc, seq);
    }

    template<> ko read_datagram(const datagram& d, t& o) {
        return o.read(d);
    }
}

using namespace us::gov::engine;
using c = us::gov::engine::rpc_peer_t;

void c::dump(const string& prefix, ostream& os) const {
    os << prefix << "engine: \n";
}

void c::dump_all(const string& prefix, ostream& os) const {
    dump(prefix, os);
    b::dump_all(prefix, os);
}

svc_t c::translate_svc(svc_t svc0, bool inbound) const {
    svc_t svc;
    if (inbound) {
        svc = daemon_t::svcfish.from_prev(svc);
        log("Using API versioning translator. oldsvc ", svc0, "-> newsvc", svc);
    }
    else {
        svc = daemon_t::svcfish.to_prev(svc);
        log("Using API versioning translator. newsvc ", svc0, "-> oldsvc", svc);
    }
    return svc;    
}

bool c::process_work(datagram* d) {
    log("process_work", d->service);
    using namespace protocol;
    /// Assuming the following svc ordering:
    /// master definition: us/api/model.cpp
    static_assert(socket_end <= id_end);
    static_assert(id_end <= auth_end);
    static_assert(auth_end <= peer_end);
    static_assert(peer_end <= relay_end);
    static_assert(relay_end <= dfs_end);
    static_assert(dfs_end <= engine_end);
    static_assert(engine_end <= cash_end);
    static_assert(cash_end <= engine_auth_end);
    static_assert(engine_auth_end <= traders_end);
    static_assert(traders_end <= sys_end);
    using namespace us::gov::protocol;
    if (d->service < net_daemon_end) {
        log("processing net_daemon datagram", d->service);
        return b::process_work(d);
    }
    return false;
}

#include <us/api/generated/gov/c++/engine/cllr_rpc-impl>
#include <us/api/generated/gov/c++/engine_auth/cllr_rpc-impl>
#include <us/api/generated/gov/c++/cash/cllr_rpc-impl>
#include <us/api/generated/gov/c++/traders/cllr_rpc-impl>
#include <us/api/generated/gov/c++/sys/cllr_rpc-impl>

