#pragma once
#include <map>
#include <set>
#include <tuple>
#include <string>
#include <string_view>
#include <type_traits>

#include "writable.h"
//#include "cfg0.h"
#include "types.h"

namespace us::gov::io {

    struct blob_writer_t;

    struct blob_writer_t {
        using writable = us::gov::io::writable;
        using version_t = writable::version_t;
        using serid_t = writable::serid_t;
        using blob_header_t = writable::blob_header_t;

    public:
        blob_writer_t(blob_t& blob, const size_t& sz);
        blob_writer_t(datagram& datagram);

    public:
        void write_header(const serid_t&);

        static string add_header(blob_header_t&&, const string& blobb58);
        static blob_t add_header(blob_header_t&&, const blob_t&);

        static size_t sizet_size(const uint64_t&);
        void write_sizet(const uint64_t&);

        template<typename t>
        static size_t blob_size(const t& o) {
            return o.blob_size();
        }

        template<typename t>
        void write(const t& o) {
            o.to_blob(*this);
        }

        template<typename t>
        static size_t blob_size(t* o) {
            if (o == nullptr) return 1;
            return 1 + o->blob_size();
        }

        template<typename t>
        void write(t* o) {
            uint8_t x = (o == nullptr ? 0 : 1);
            write(x);
            if (x == 1) {
                write(*o);
            }
        }


        template<typename t>
        static size_t blob_size(t* o, const factories_t<t>& f) {
            if (o == nullptr) return blob_size(t::null_instance);
            return blob_size(o->factory_id()) + o->blob_size();
        }

        template<typename t>
        void write(t* o, const factories_t<t>& f) {
//cout << "wcur=" << (uint64_t)(cur - blob.data()) << endl;
            typename t::factory_id_t x = (o == nullptr ? t::null_instance : o->factory_id());
            write(x);
            if (o == nullptr) {
                return;
            }
            write(*o);
        }

        static size_t blob_size(const char* s);
        void write(const char* s);
/*
        static datagram* get_datagram(channel_t, svc_t svc, seq_t seq);

        template<typename t>
        static datagram* get_datagram(channel_t channel, svc_t svc, seq_t seq, const t& o) {
            static_assert(!std::is_convertible<t*, writable*>::value, "KO 77897 template specialization for writable should have been instantiated instead.");
            static_assert(!std::is_convertible<t*, blob_t*>::value, "KO 77898 template specialization for blob_b should have been instantiated instead.");

            auto d = new datagram(channel, svc, seq, blob_size(o));
            blob_writer_t w(*d);
            w.write(o);
            return d;
        }
*/
        static blob_t make_blob(const string& payload);
        static blob_t make_blob(const hash_t& payload);

        //-------------------------

        template<typename t>
        static size_t blob_size(const vector<t>& o) {
            size_t sz = sizet_size(o.size());
            for (auto& i: o) {
                sz += blob_size(i);
            }
            return sz;
        }

        template<typename t>
        void write(const vector<t>& o) {
            write_sizet(o.size());
            for (auto& i: o) {
                write(i);
            }
        }

        template<typename t1, typename t2>
        static size_t blob_size(const pair<t1, t2>& o) {
            return blob_size(o.first) + blob_size(o.second);
        }

        template<typename t1, typename t2>
        void write(const pair<t1, t2>& o) {
            write(o.first);
            write(o.second);
        }


        template<typename t1, typename t2, typename t3>
        static size_t blob_size(const tuple<t1, t2, t3>& o) {
            return blob_size(get<0>(o)) + blob_size(get<1>(o)) + blob_size(get<2>(o));
        }

        template<typename t1, typename t2, typename t3>
        void write(const tuple<t1, t2, t3>& o) {
            write(get<0>(o));
            write(get<1>(o));
            write(get<2>(o));
        }

        template<typename k, typename v>
        static size_t blob_size(const map<k, v>& o) {
            size_t sz = sizet_size(o.size());
            for (auto& i: o) {
                sz += blob_size(i.first);
                sz += blob_size(i.second);
            }
            return sz;
        }

        template<typename k, typename v>
        void write(const map<k, v>& o) {
            write_sizet(o.size());
            for (auto& i: o) {
                write(i.first);
                write(i.second);
            }
        }

        template<typename k, typename v>
        static size_t blob_size(const unordered_map<k, v>& o) {
            size_t sz = sizet_size(o.size());
            for (auto& i: o) {
                sz += blob_size(i.first);
                sz += blob_size(i.second);
            }
            return sz;
        }

        template<typename k, typename v>
        void write(const unordered_map<k, v>& o) {
            write_sizet(o.size());
            for (auto& i: o) {
                write(i.first);
                write(i.second);
            }
        }

        template<typename k>
        static size_t blob_size(const set<k>& o) {
            size_t sz = sizet_size(o.size());
            for (auto& i: o) {
                sz += blob_size(i);
            }
            return sz;
        }

        template<typename k>
        void write(const set<k>& o) {
            write_sizet(o.size());
            for (auto& i: o) {
                write(i);
            }
        }

        blob_t& blob;
        uint8_t* cur;
    };

//    template<> datagram* blob_writer_t::get_datagram(channel_t, svc_t, seq_t, const blob_t&);
//    template<> datagram* blob_writer_t::get_datagram(channel_t, svc_t, seq_t, const writable&);

    template<> constexpr size_t blob_writer_t::blob_size(const bool&) { return sizeof(uint8_t); }
    template<> void blob_writer_t::write(const bool&);

    template<> constexpr size_t blob_writer_t::blob_size(const uint8_t&) { return sizeof(uint8_t); }
    template<> void blob_writer_t::write(const uint8_t&);

    template<> constexpr size_t blob_writer_t::blob_size(const uint16_t&) { return sizeof(uint16_t); }
    template<> void blob_writer_t::write(const uint16_t&);

    template<> constexpr size_t blob_writer_t::blob_size(const uint32_t&) { return sizeof(uint32_t); }
    template<> void blob_writer_t::write(const uint32_t&);

    template<> constexpr size_t blob_writer_t::blob_size(const uint64_t&) { return sizeof(uint64_t); }
    template<> void blob_writer_t::write(const uint64_t&);

    template<> constexpr size_t blob_writer_t::blob_size(const int64_t&) { return sizeof(int64_t); }
    template<> void blob_writer_t::write(const int64_t&);

    template<> constexpr size_t blob_writer_t::blob_size(const int32_t&) { return sizeof(int32_t); }
    template<> void blob_writer_t::write(const int32_t&);

    template<> size_t blob_writer_t::blob_size(const string&);
    template<> void blob_writer_t::write(const string&);

    template<> size_t blob_writer_t::blob_size(const string_view&);
    template<> void blob_writer_t::write(const string_view&);

    template<> size_t blob_writer_t::blob_size(const blob_t&);
    template<> void blob_writer_t::write(const blob_t&);

    template<> constexpr size_t blob_writer_t::blob_size(const hash_t&) { return hasher_t::output_size; }
    template<> void blob_writer_t::write(const hash_t&);

    template<> constexpr size_t blob_writer_t::blob_size(const sigmsg_hash_t&) { return sigmsg_hasher_t::output_size; }
    template<> void blob_writer_t::write(const sigmsg_hash_t&);

    template<> constexpr size_t blob_writer_t::blob_size(const pub_t&) { return pub_t::ser_size; }
    template<> void blob_writer_t::write(const pub_t&);

    template<> constexpr size_t blob_writer_t::blob_size(const priv_t&) { return priv_t::ser_size; }
    template<> void blob_writer_t::write(const priv_t&);

    template<> constexpr size_t blob_writer_t::blob_size(const keys& o) { return blob_size(o.priv) + blob_size(o.pub); }
    template<> void blob_writer_t::write(const keys&);

    template<> size_t blob_writer_t::blob_size(const sig_t&);
    template<> void blob_writer_t::write(const sig_t&);

    template<> constexpr size_t blob_writer_t::blob_size(const time_point&) { return sizeof(uint64_t); }
    template<> void blob_writer_t::write(const time_point&);

    template<> constexpr size_t blob_writer_t::blob_size(const char&) { return sizeof(char); }
    template<> void blob_writer_t::write(const char&);

    inline size_t blob_writer_t::blob_size(const char* s) { return blob_size(string_view(s)); }
    inline void blob_writer_t::write(const char* s) { write(string_view(s)); }

}

