#include <thread>

#include <us/gov/config.h>

#include "types.h"
#include "params.h"

//#define loglevel "gov/io"
//#define logclass "params"
//#include "logs.inc"

// don't log here, it's used before initializing the logging subsystem

using namespace us::gov::io;
using c = us::gov::io::params;

c::params(): args("") {
    workers = 2 * thread::hardware_concurrency();
}

c::params(const shell_args& a): args(a) {
    workers = 2 * thread::hardware_concurrency();

    while(true) {
        string command = args.next<string>();
        if (command == "-p") {
            port = args.next<port_t>();
        }
        else if (command == "-pp") {
            pport = args.next<pport_t>();
        }
        else if (command == "-d") {
            daemon = true;
        }
        else if (command == "-ds") {
            daemon = true;
            shell = true;
        }
        else if (command == "-fs") {
            force_seeds = true;
        }
        else if (command == "-dot") {
            dot = true;
        }
        else if (command == "-e") {
            edges = args.next<int>();
        }
        else if (command == "-E") {
            save_evidences = (args.next<int>(0) == 1);
        }
        else if (command == "-v") {
            devices = args.next<int>();
        }
        else if (command == "-w") {
            workers = args.next<int>();
        }
        else if (command == "-c") {
            channel = args.next<channel_t>();
        }
        else if (command == "-home") {
            homedir = args.next<string>();
        }
        else if (command == "-host") {
            sysophost = args.next<string>();
        }
        else if (command == "-xp") {
            //daemon_encrypt_protocol = args.next<bool>(false); //Ignored. deprecated
        }
        else if (command == "-status") {
            status_file = args.next<bool>(false);
        }
        #if CFG_LOGS == 1
        else if (command == "-log") {
            verbose = true;
        }
        else if (command == "-logd") {
            logd = args.next<string>();
        }
        #endif

        else if (command == "-om") {
            uint16_t m = args.next<uint16_t>();
            if (m < num_modes) om = (output_mode)m;
        }
        else if (command == "-n") {
            names = true;
        }
        else if (command == "-nb") {
            banner = false;
        }
        else if (command == "-t") {
            rpc_timeout_secs = args.next<uint16_t>(CFG_DGRAM_ROUNDTRIP_TIMEOUT_SECS);
        }
        else if (!command.empty()) {
            cmd = command + " " + args.next_line();
            break;
        }
        else {
            break;
        }
    }
}

string c::get_home_gov() const {
    return get_home() + "/gov";
}

string c::get_home_gov_rpc_client() const {
    return get_home_gov() + "/rpc_client";
}

string c::get_status_file() const {
    if (!status_file) return "";
    return get_home_gov() + "/status";
}

string c::get_home() const {
    ostringstream os;
    if (homedir.empty()) {
        const char* env_p = std::getenv("HOME");
        if (!env_p) {
            cerr << "No $HOME env var defined" << endl;
            exit(1);
        }
        os << env_p << "/" << DATADIR;
    }
    else {
        os << homedir;
    }
    if (channel != 0) {
        os << '/' << channel;
    }
    return os.str();
}

void c::dump(ostream& os) const {
    dump("", os);
}

void c::dump(const string& pfx, ostream& os) const {
    os << pfx << "channel: " << channel << '\n';
    os << pfx << "home: " << get_home() << '\n';
    #ifdef DEBUG
        os << pfx << "warning: this is a debug build.\n";
    #else
        os << pfx << "this is an optimized build.\n";
    #endif
    #if CFG_LOGS == 1
        if (verbose) {
            os << pfx << "writing log to cout\n";
        }
        else {
            os << pfx << "warning: logs are ON. Writing files in " << logd << '\n';
        }
    #else
        os << pfx << "logs: disabled.\n";
    #endif
    if (daemon) {
        os << pfx << "run daemon\n";
        os << pfx << "  listening port: " << port << '\n';
        os << pfx << "  published listening port: " << pport << '\n';
        os << pfx << "  network topology: " << TOPOLOGY << '\n';
        os << pfx << "  max node edges: " << (int)edges << '\n';
        os << pfx << "  max wallet edges: " << (int)devices << '\n';
//        os << pfx << "  protocol is " << (daemon_encrypt_protocol ? "" : "not ") << "encrypted\n";
        os << pfx << "  workers: " << (int)workers << '\n';
        os << pfx << "  sysop shell: " << boolalpha << shell << '\n';
    }
    else {
        os << pfx << "run rpc shell\n";
        os << pfx << "  host: " << sysophost << ":" << port << '\n';
        os << pfx << "  connect_for_recv " << rpc__connect_for_recv << '\n';
        os << pfx << "  stop_on_disconnection " << rpc__stop_on_disconnection << '\n';
    }
}

void c::connecting_to(ostream& os) const {
    os << PLATFORM << "-gov daemon at " << sysophost << ':' << port;
}

