#pragma once
#include <vector>
#include <thread>
#include <iostream>
#include <algorithm>
#include <sstream>
#include <chrono>
#include <cassert>
#include <array>
#include <unordered_set>
#include <us/gov/config.h>
#include <us/gov/auth/peer_t.h>
#include "api.h"
#include "types.h"

namespace us::gov::peer {

    struct daemon_t;

    struct peer_t: base_ns::peer_t, api {
        using b = base_ns::peer_t;

        enum stage_t {
            stage_disconnected = 0,
            stage_connected,
            stage_exceed_latency,
            stage_service,
            stage_disconnecting,
            num_stages
        };
        constexpr static array<const char*, num_stages> stagestr = {"disconnected", "connected", "latency", "service", "disconnecting"};
        static chrono::seconds auth_window;
        constexpr static ts_t maxidle_ms{180000};
        static const char* finished_reason_1;

    public:
        peer_t(daemon_t&, sock_t sock);
        ~peer_t() override;

    public:
        ko connect(const hostport_t&, pport_t, pin_t, role_t, const request_data_t&, bool block) final override;
        using b::disconnect;
        void disconnectx(channel_t, seq_t, const reason_t&) override;
        void dump_all(const string& prefix, ostream&) const override;
        pair<security_level_t, string> security_level() const override;
        bool check_auth_not_granted() const;
        bool check_idle() const;
        void dump(const string& prefix, ostream&) const;
        ko authorize(const pub_t&, pin_t, request_data_t&) override;
        bool process_work(datagram*) override;

    public:
        #ifdef has_us_gov_peer_api
            #include <us/api/generated/gov/c++/peer/hdlr_override>
            #include <us/api/generated/gov/c++/peer/hdlr_svc_handler-hdr>
            #include <us/api/generated/gov/c++/peer/cllr_override>
        #endif

    public:
        stage_t stage;
    };

}

