package us.gov.crypto;
import java.util.Arrays;                                                                       // Arrays
import org.bouncycastle.asn1.ASN1Integer;                                                      // ASN1Integer
import java.math.BigInteger;                                                                   // BigInteger
import org.bouncycastle.util.BigIntegers;                                                      // BigIntegers
import org.bouncycastle.jce.provider.BouncyCastleProvider;                                     // BouncyCastleProvider
import java.io.ByteArrayOutputStream;                                                          // ByteArrayOutputStream
import us.CFG;                                                                                 // CFG
import org.bouncycastle.asn1.DERSequenceGenerator;                                             // DERSequenceGenerator
import org.bouncycastle.math.ec.ECCurve;                                                       // ECCurve
import org.bouncycastle.crypto.params.ECDomainParameters;                                      // ECDomainParameters
import org.bouncycastle.jce.spec.ECNamedCurveParameterSpec;                                    // ECNamedCurveParameterSpec
import org.bouncycastle.jce.spec.ECNamedCurveSpec;                                             // ECNamedCurveSpec
import org.bouncycastle.jce.ECNamedCurveTable;                                                 // ECNamedCurveTable
import org.bouncycastle.jce.spec.ECParameterSpec;                                              // ECParameterSpec
import org.bouncycastle.math.ec.ECPoint;                                                       // ECPoint
import org.bouncycastle.jce.ECPointUtil;                                                       // ECPointUtil
import org.bouncycastle.jce.interfaces.ECPrivateKey;                                           // ECPrivateKey
import org.bouncycastle.jce.spec.ECPrivateKeySpec;                                             // ECPrivateKeySpec
import org.bouncycastle.jce.interfaces.ECPublicKey;                                            // ECPublicKey
import org.bouncycastle.jce.spec.ECPublicKeySpec;                                              // ECPublicKeySpec
import java.security.GeneralSecurityException;                                                 // GeneralSecurityException
import org.bouncycastle.util.encoders.Hex;                                                     // Hex
import static us.gov.crypto.types.*;                                                           // *
import java.security.InvalidAlgorithmParameterException;                                       // InvalidAlgorithmParameterException
import java.security.InvalidKeyException;                                                      // InvalidKeyException
import java.security.spec.InvalidKeySpecException;                                             // InvalidKeySpecException
import java.io.IOException;                                                                    // IOException
import java.security.KeyFactory;                                                               // KeyFactory
import java.security.KeyPairGenerator;                                                         // KeyPairGenerator
import java.security.KeyPair;                                                                  // KeyPair
import us.ko;                                                                                  // ko
import java.security.NoSuchAlgorithmException;                                                 // NoSuchAlgorithmException
import java.security.NoSuchProviderException;                                                  // NoSuchProviderException
import static us.ko.ok;                                                                        // ok
import java.security.PrivateKey;                                                               // PrivateKey
import java.security.Provider;                                                                 // Provider
import java.security.PublicKey;                                                                // PublicKey
import java.security.Security;                                                                 // Security
import org.bouncycastle.crypto.digests.SHA256Digest;                                           // SHA256Digest
import java.security.Signature;                                                                // Signature

public class ec {
    linux_secure_random secure_random;
    KeyFactory factory;
    ECParameterSpec spec;
    ECDomainParameters curve;
    KeyPairGenerator generator;
    Signature dsa;

    public static ec instance = null;


    public static void create_instance() {
        if (instance != null) {
            return;
        }
        try {
            Provider p = new BouncyCastleProvider();
            if (p == null) {
                System.err.println("KO 10199 BC Provider not found.");
                System.exit(1);
            }
            Security.insertProviderAt(p, 1);
            instance = new ec("secp256k1", "ECDSA", "NONEwithECDSA");
        }
        catch (Exception e) {
            System.out.println("KO 10099 ECDSA: " + e.getMessage());
            System.exit(1);
        }
    }

    public ec(String curve_name, String algorithm, String signature_algorithm) throws GeneralSecurityException {
        dsa = Signature.getInstance(signature_algorithm);
        spec = ECNamedCurveTable.getParameterSpec(curve_name);
        factory = KeyFactory.getInstance(algorithm);
        curve = new ECDomainParameters(spec.getCurve(), spec.getG(), spec.getN(), spec.getH());
        secure_random = new linux_secure_random();
        generator = KeyPairGenerator.getInstance(algorithm);
        generator.initialize(spec);
    }

    public BigInteger generate_private_int() {
        KeyPair keypair = generate_keypair();
        if (keypair == null) return null;
        return get_private_int(keypair);
    }

    public BigInteger get_private_int(PrivateKey k0) {
        ECPrivateKey k = (ECPrivateKey) k0;
        return k.getD();
    }

    private BigInteger get_private_int(KeyPair keypair) {
        return get_private_int(keypair.getPrivate());
    }

    public PrivateKey get_private_key(BigInteger k) {
        try {
            ECPrivateKeySpec ks = new ECPrivateKeySpec(k, spec);
            return factory.generatePrivate(ks);
        }
        catch (Exception e) {
            return null;
        }
    }

    public KeyPair generate_keypair() {
        try {
            return generator.generateKeyPair();
        }
        catch (Exception e) {
            return null;
        }
    }

    public KeyPair generate_keypair(final String sk_b58) {
        return generate_keypair(get_private_key(sk_b58));
    }

    public KeyPair generate_keypair(PrivateKey k) {
        try {
            return new KeyPair(get_public_key(k), k);
        }
        catch (Exception e) {
            return null;
        }
    }

    public KeyPair get_keypair(BigInteger priv) {
        return generate_keypair(get_private_key(priv));
    }

    public byte[] generate_shared_key(PrivateKey priv, PublicKey pub, int length) {
        try {
            ECPrivateKey k = (ECPrivateKey) priv;
            ECPublicKey p = (ECPublicKey) pub;
            sha256.hash_t h = sha256.hash(multiply(p.getQ(), k.getD()).getEncoded(true));
            return Arrays.copyOf(h.value, 16);
        }
        catch (Exception e) {
            return null;
        }
    }

    public byte[] to_vector(PublicKey pub) {
        try {
            ECPublicKey p = (ECPublicKey) pub;
            ECPoint point = p.getQ();
            return point.getEncoded(true);
        }
        catch (Exception e) {
            return null;
        }
    }

    public byte[] to_vector(PrivateKey priv) {
        try {
            ECPrivateKey k = (ECPrivateKey) priv;
            return BigIntegers.asUnsignedByteArray(32, k.getD());
        }
        catch (Exception e) {
            return null;
        }
    }

    public ripemd160.hash_t to_address(PublicKey k) {
        return ripemd160.hash(to_vector(k));
    }

    public String to_encoded_address(PublicKey k) {
        return base58.encode(to_address(k).value);
    }

    public String to_b58(PublicKey k) {
            return base58.encode(to_vector(k));
    }

    public String to_b58(PrivateKey k) {
        try {
            return base58.encode(to_vector(k));
        }
        catch (Exception e) {
            return null;
        }
    }

    public PrivateKey get_private_key(String b58) {
        try {
            return get_private_key(base58.decode(b58));
        }
        catch (Exception e) {
            return null;
        }
    }

    public PrivateKey get_private_key(byte[] privateKey) {
        try {
            return get_private_key(BigIntegers.fromUnsignedByteArray(privateKey));
        }
        catch (Exception e) {
            return null;
        }
    }

    public PrivateKey get_private_key(byte[] buffer, int offset) {
        byte[] v = new byte[32];
        System.arraycopy(buffer, offset, v, 0, 32);
        return get_private_key(BigIntegers.fromUnsignedByteArray(v));
    }

    private ECPoint get_point(byte[] pub) {
        return spec.getCurve().decodePoint(pub);
    }

    public ECPoint multiply(ECPoint point, BigInteger d) {
        if (d.bitLength() > spec.getN().bitLength()) {
            d = d.mod(spec.getN());
        }
        return point.multiply(d);
    }

    public PublicKey get_public_key(BigInteger priv) {
        try {
            ECPoint point = multiply(spec.getG(), priv);
            return get_public_key(point);
        }
        catch (Exception e) {
            return null;
        }
    }

    public PublicKey get_public_key(PrivateKey priv) {
        try {
            BigInteger i = get_private_int(priv);
            return get_public_key(i);
        }
        catch (Exception e) {
            return null;
        }
    }

    public PublicKey get_public_key(ECPoint point) {
        try {
            ECPublicKeySpec s = new ECPublicKeySpec(point, spec);
            return factory.generatePublic(s);
        }
        catch (Exception e) {
            return null;
        }
    }

    public PublicKey get_public_key(byte[] public_key) {
        try {
            ECPoint point = get_point(public_key);
            return get_public_key(point);
        }
        catch (Exception e) {
            return null;
        }
    }

    public PublicKey get_public_key(byte[] buffer, int offset) {
        assert buffer.length >= offset + 33;
        byte[] v = new byte[33];
        System.arraycopy(buffer, offset, v, 0, 33);
        return get_public_key(v);
    }

    public boolean verify(PublicKey pub, sha256.hash_t msgh, sig_der_t sig_der) {
        try {
            dsa.initVerify(pub);
            dsa.update(msgh.value);
            return dsa.verify(sig_der.value);
        }
        catch (Exception e) {
            return false;
        }
    }

    public sig_der_t sign(PrivateKey priv, sha256.hash_t msgh) {
        try {
            dsa.initSign(priv);
            dsa.update(msgh.value);
            byte[] r = dsa.sign();
            return new sig_der_t(r);
        }
        catch (Exception e) {
            return null;
        }
    }

}

