package us.gov.socket.outbound;
import java.util.ArrayList;                                                                    // ArrayList
import java.util.Arrays;                                                                       // Arrays
import us.gov.bgtask;                                                                          // bgtask
import us.gov.socket.busyled_t;                                                                // busyled_t
import us.CFG;                                                                                 // CFG
import us.gov.socket.client;                                                                   // client
import java.lang.Comparable;                                                                   // Comparable
import java.util.concurrent.locks.Condition;                                                   // Condition
import us.gov.socket.datagram;                                                                 // datagram
import java.util.HashMap;                                                                      // HashMap
import static us.gov.socket.types.*;                                                           // *
import us.stdint.*;                                                                            // *
import java.lang.InterruptedException;                                                         // InterruptedException
import java.io.IOException;                                                                    // IOException
import us.ko;                                                                                  // ko
import java.util.concurrent.locks.Lock;                                                        // Lock
import static us.ko.ok;                                                                        // ok
import us.pair;                                                                                // pair
import java.io.PrintStream;                                                                    // PrintStream
import java.util.PriorityQueue;                                                                // PriorityQueue
import java.util.concurrent.locks.ReentrantLock;                                               // ReentrantLock
import java.net.SocketException;                                                               // SocketException

@SuppressWarnings("serial")
public class send_queue_t extends bgtask implements bgtask.callback {

    public static int wmh = CFG.SENDQ_WMH;
    public static int schunk = CFG.SENDQ_SCHUNK;


    public static class qi extends pair<datagram, client> implements Comparable<qi> {
        public qi(datagram d, client c) {
            super(d, c);
            first.dend = 0;
            second.sendref.incrementAndGet();
        }

        public void destroy() {
            first = null;
            second.sendref.decrementAndGet();
        }

        public int compareTo(qi other) {
            if (first.service.value == other.first.service.value) return 0;
            return first.service.value > other.first.service.value? -1 : 1;
        }

        public static long prev_uid = 0;
        public long uid = ++prev_uid;
    }

    public static class ongoing_t extends HashMap<Long, pair<client, qi>>  {
        public void set(qi x) {
            pair<client, qi> i = get(x.second.uid);
            if (i != null) {
                return;
            }
            put(x.second.uid, new pair<client, qi>(x.second, x));
        }

        public qi reset(qi x) {
            pair<client,qi> i = get(x.second.uid);
            if (i == null) return null;
            remove(x.second.uid);
            return x;
        }
    };

    public send_queue_t() {
         set_callback(this);
    }

    public void onwakeup() {
        lock.lock();
        try {
            cv.signalAll();
        }
        finally {
            lock.unlock();
        }
    }

    public void clear() {
        lock.lock();
        try {
            ongoing.clear();
            q.clear();
        }
        finally {
            lock.unlock();
        }
    }

    public void dump(final String prefix, PrintStream os) {
        lock.lock();
        try {
            os.println(prefix + "active " + is_active());
            os.println(prefix + "wmh " + wmh);
            os.println(prefix + "schunk " + schunk);
            os.println(prefix + "queued " + q.size());
        }
        finally {
            lock.unlock();
        }
    }

    int get_size() {
        lock.lock();
        int s = 0;
        try {
            s = q.size();
        }
        finally {
            lock.unlock();
        }
        return s;
    }

    void flood(datagram d, client cli) {
    }

    void send(datagram d, client cli, int pri) {
        assert d.completed();
        d.service.value = pri;
        String ret = null;
        lock.lock();
        try {
            if (q.size() >= wmh) {
                qi o = q.poll();
                flood(o.first, o.second);
            }
            q.add(new qi(d, cli));
            task_wakeup();
            assert q.size() < wmh;
        }
        finally {
            lock.unlock();
        }
    }

    void purge(client c) {
        lock.lock();
        try {
            ArrayList<qi> v = new ArrayList<qi>(q.size());
            while(!q.isEmpty()) {
                qi x = q.poll();
                if (x.second.uid == c.uid) {
                    ongoing.reset(x);
                    continue;
                }
                v.add(x);
            }
            for (qi i:v) {
                q.add(i);
            }
        }
        finally {
            lock.unlock();
        }
    }

    qi next() {
        qi ans = null;
        lock.lock();
        try {
            ans = next_();
        }
        finally {
            lock.unlock();
        }
        return ans;

    }

    qi next_() {
        while (true) {
            if(q.isEmpty()) {
                reset_wait();
                return null;
            }
            qi i = q.peek();
            if (i.first.size() == i.first.dend) {
                q.poll();
                ongoing.reset(i);
                continue;
            }
            //avoid starting a new dgram for a client that has another ongoing
            qi r = replace(i); //same i (if no other qi has been started for this client), othersise the unfinished qi
            return r;
        }
    }

    qi replace(qi x) {
        assert x != null;
        pair<client,qi> i = ongoing.get(x.second.uid);
        if (i != null) {
            if (i.second.first.dend == i.second.first.size()) {
                ongoing.remove(i);
                return x;
            }
            return i.second;
        }
        return x;
    }

    public void run_body() {
        while (isup()) {
            qi d = next();
            if (d == null) {
                await();
                continue;
            }
            int rem = d.first.size() - d.first.dend;
            int sz = rem < 2 * schunk ? rem : schunk;
            if (d.second.sendref.get() < 0) {
                d.first.dend = d.first.size();
                continue;
            }
            if (d.second.sock == null) {
                d.first.dend = d.first.size();
                continue;
            }

            byte[] chunk = Arrays.copyOfRange(d.first.bytes, d.first.dend, d.first.dend+sz);
            boolean wrong = false;
            busyled.set();
            try {
                //The write method of OutputStream calls the write method of one argument on each of the bytes to be written out. Subclasses are encouraged to override this method and provide a more efficient implementation.
                d.second.sock.getOutputStream().write(chunk);
            }
            catch (SocketException e) {
                wrong = true;
            }
            catch (IOException e) {
                wrong = true;
            }
            catch (Exception e) {
                wrong = true;
            }
            busyled.reset();
            if (!wrong) {
                d.first.dend += sz;
                if (d.first.in_progress()) {
                    ongoing.set(d);
                }
                else {
                    ongoing.reset(d);
                }
                continue;
            }
            else {
                d.second.disconnect(new seq_t(0), new reason_t());
                d.first.dend = d.first.size(); //finish with this dgram
            }
        }
    }

    public void await() {
        if (reset_wait()) {
            return;
        }
        lock.lock();
        try {
            while (!resumed()) {
                cv.await();
            }
        }
        catch (InterruptedException e) {
        }
        finally {
            lock.unlock();
        }
        reset_wait();
    }

    public void set_busy_handler(busyled_t.handler_t hsend) {
        busyled.set_handler(hsend);
    }

    PriorityQueue<qi> q = new PriorityQueue<qi>(CFG.SENDQ_SIZE);
    Lock lock = new ReentrantLock();
    Condition cv = lock.newCondition();
    ongoing_t ongoing = new ongoing_t();
    public busyled_t busyled = new busyled_t();

}

