package us.gov.socket;
import us.gov.io.blob_reader_t;                                                                // blob_reader_t
import us.gov.io.blob_writer_t;                                                                // blob_writer_t
import us.CFG;                                                                                 // CFG
import static us.gov.socket.types.*;                                                           // *
import us.stdint.*;                                                                            // *
import us.ko;                                                                                  // ko
import static us.ko.ok;                                                                        // ok
import us.pair;                                                                                // pair
import java.io.PrintStream;                                                                    // PrintStream
import us.gov.protocol;                                                                        // protocol
import java.net.Socket;                                                                        // Socket
import us.string;                                                                              // string

public abstract class peer_t extends client implements api {

    public static final ko KO_50143 = new ko("KO 50143 Unexpected service number.");
    public static final ko KO_20179 = new ko("KO 20179 Ignored.");
    public static final ko KO_10210 = new ko("KO 10210 Not implemented.");


    public peer_t(rpc_daemon_t rpc_daemon) {
        super(rpc_daemon);
    }

    public peer_t(rpc_daemon_t rpc_daemon, Socket sock) {
        super(rpc_daemon, sock);
    }

    public void ping() {
        string pong = new string();
        ko r = call_ping(new string("ping"), pong);
    }

    public void on_peer_disconnected(reason_t reason) {
    }

    @Override public void dump(final String prefix, PrintStream os) {
        os.println(prefix + "socket::peer_t: idle(recv) " + age(activity_recv));;
        super.dump(prefix, os);
    }

    public void prepare_worker_recv() {
        daemon.recv_led().set();
        //++busyrecv;
    }

    public void worker_ended_recv() {
        //--busyrecv;
        daemon.recv_led().reset();
    }

    public ko send1(datagram d) {
        if (!daemon.is_active()) {
            ko r = new ko("KO 73101 daemon is inactive.");
            return r;
        }
        return daemon.send1(this, d);
    }

    public pair<ko, datagram> sendrecv(datagram d, string remote_error) {
        return daemon.sendrecv(this, d, remote_error);
    }

    public void process_ok_work(datagram d) {
        daemon.process_ok_work(this, d);
    }

    public void process_ko_work(final channel_t channel, final seq_t seq, ko r) {
        daemon.process_ko_work(this, channel, seq, r);
    }

    @Override public void set_finished() {
        super.set_finished();
    }

    public boolean process_work(datagram d) {
        assert d != null;
        assert d.service.value < protocol.socket_end;
        switch(d.service.value) {
            //#include <us/api/generated/gov/java/socket/hdlr_svc-router>
            //------------------------------------------------------------__begin__------generated by configure, do not edit.
            //content of file: <us/api/generated/gov/java/socket/hdlr_svc-router>
            // socket - master file: us/api/data/gov/socket

            case protocol.socket_error: { return process_async_api__socket_error(d); }
            case protocol.socket_ping: { return process_sync_api__socket_ping(d); }
            case protocol.socket_finished: { return process_async_api__socket_finished(d); }
            //-/----------------------------------------------------------___end___------generated by configure, do not edit.
        }
        return false;
    }

    //#include <us/api/generated/gov/java/socket/hdlr_svc_handler-impl>
    //------------------------------------------------------------__begin__------generated by configure, do not edit.
    //content of file: <us/api/generated/gov/java/socket/hdlr_svc_handler-impl>
    // socket - master file: us/api/data/gov/socket

    boolean process_async_api__socket_error(datagram d) {
        string msg = new string();
        {
            ko r = blob_reader_t.readD(d, msg);
            if (ko.is_ko(r)) {
                return true; //processed
            }
        }
        {
            ko r = handle_error(d, msg);
            if (ko.is_ko(r)) {
                process_ko_work(d.decode_channel(), d.decode_sequence(), r);
                return true; //processed
            }
        }
        return true; //processed
    }

    boolean process_sync_api__socket_ping(datagram d) {
        string msg = new string();
        {
            ko r = blob_reader_t.readD(d, msg);
            if (ko.is_ko(r)) {
                return true; //processed
            }
        }
        string ans = new string();
        {
            ko r = handle_ping(msg, ans);
            if (ko.is_ko(r)) {
                process_ko_work(d.decode_channel(), d.decode_sequence(), r);
                return true; //processed
            }
        }
        datagram dout = blob_writer_t.get_datagram(d.decode_channel(), new svc_t(protocol.socket_ping_response), d.decode_sequence(), ans);
        process_ok_work(dout);
        return true; //processed
    }

    boolean process_async_api__socket_finished(datagram d) {
        string reason = new string();
        {
            ko r = blob_reader_t.readD(d, reason);
            if (ko.is_ko(r)) {
                return true; //processed
            }
        }
        {
            ko r = handle_finished(reason);
            if (ko.is_ko(r)) {
                process_ko_work(d.decode_channel(), d.decode_sequence(), r);
                return true; //processed
            }
        }
        return true; //processed
    }
    //-/----------------------------------------------------------___end___------generated by configure, do not edit.

//------------------apitool - API Spec defined @ us/api/generated/gov/java/socket/hdlr_local-impl

@Override public ko handle_error(final datagram dgram, final string msg) {
    daemon.rendezvous.arrived(dgram);
    return ok;
}

@Override public ko handle_ping(final string msg, string ans) {
    ans.value = "pong";
    return ok;
}

@Override public ko handle_finished(final string reason) {
    on_peer_disconnected(new reason_t(reason.value));
    return ok;
}

//-/----------------apitool - End of API implementation.



    //#include <us/api/generated/gov/java/socket/cllr_rpc-impl>
    //------------------------------------------------------------__begin__------generated by configure, do not edit.
    //content of file: <us/api/generated/gov/java/socket/cllr_rpc-impl>
    @Override public ko call_error(final string msg) {
        return send1(blob_writer_t.get_datagram(daemon.channel, new svc_t(protocol.socket_error), new seq_t(0), msg));
    }

    @Override public ko call_ping(final string msg, string ans) {
        datagram d_in;
        {
            string lasterr = new string();
            pair<ko, datagram> r = sendrecv(blob_writer_t.get_datagram(daemon.channel, new svc_t(protocol.socket_ping), new seq_t(0), msg), lasterr);
            if (ko.is_ko(r.first)) {
                assert r.second == null;
                if (lasterr.value != null) {
                    lasterror.set(lasterr.value);
                }
                else {
                    lasterror.set("");
                }
                return r.first;
            }
            assert r.second != null;
            d_in = r.second;
        }
        if (d_in.service.value != protocol.socket_ping_response) {
            ko r = KO_50143;
            return r;
        }
        ko r = blob_reader_t.readD(d_in, ans);
        return r;
    }

    @Override public ko call_finished(final string reason) {
        return send1(blob_writer_t.get_datagram(daemon.channel, new svc_t(protocol.socket_finished), new seq_t(0), reason));
    }
    //-/----------------------------------------------------------___end___------generated by configure, do not edit.

    public static ThreadLocal<String> lasterror = new ThreadLocal<String>();

}

