package us.gov.socket;
import us.CFG;                                                                                 // CFG
import java.util.Date;                                                                         // Date
import static us.gov.socket.datagram.dispatcher_t;                                             // dispatcher_t
import us.gov.socket.single.handler_daemon_t;                                                  // handler_daemon_t
import static us.gov.socket.types.*;                                                           // *
import java.io.IOException;                                                                    // IOException
import static us.ko.is_ko;                                                                     // is_ko
import us.ko;                                                                                  // ko
import static us.ko.ok;                                                                        // ok
import us.pair;                                                                                // pair
import java.io.PrintStream;                                                                    // PrintStream
import us.gov.socket.outbound.rendezvous_t;                                                    // rendezvous_t

public abstract class rpc_daemon_t extends daemon0_t {


    public rpc_daemon_t(channel_t channel, dispatcher_t dispatcher) {
        super(channel);
        handler = new handler_daemon_t(this, dispatcher);
    }

    public boolean sendq_active() {
        return is_active();
    }

    @Override public ko send1(client peer, datagram d) {
        ko r = wait_recv_connection(true);
        if (ko.is_ko(r)) {
            return r;
        }
        return super.send1(peer, d);
    }

    public void on_destroy_(client peer) {}
    public abstract ko connect();

    public ko wait_recv_connection(boolean initiate) {
        return handler.wait_connection(initiate);
    }

    public client create_client() {
//        return new peer_t(this);
        assert false;
        return null;
    }

    @Override public final boolean process_unhandled(client peer, datagram d) {
        if (super.process_unhandled(peer, d)) {
            return true;
        }
        return handler.process_unhandled(peer, d);
    }

    public void set_busy_handlers(busyled_t.handler_t hsend, busyled_t.handler_t hrecv) {
        handler.set_busy_handler(hrecv);
        super.set_busy_handler(hsend);
    }

    @Override public void dump(final String prefix, PrintStream os) {
        os.println(prefix + "socket::rpc_daemon_t:");
        super.dump(prefix, os);
    }

    @Override public final busyled_t recv_led() {
        return handler.busyled;
    }

    public void wakeup_handler() {
        handler.task_wakeup();
    }

    public void wakeup_caller() {
        super.task_wakeup();
    }

    @Override public String rewrite(ko r) {
        if (r == rendezvous_t.KO_20190) {
            String x = peer_t.lasterror.get();
            if (x != null) {
                return x;
            }
            return r.msg;
        }
        return super.rewrite(r);
    }

    public ko start() {
        assert peer == null;
        client o = create_client();
        attach(o);
        {
            ko r = super.start();
            if (ko.is_ko(r)) {
                return r;
            }
        }
        {
            ko r = handler.start();
            if (ko.is_ko(r)) {
                super.stop();
                return r;
            }
        }
        return ok;
    }

    public ko wait_ready(Date deadline) {
        ko r = handler.wait_ready(deadline);
        if (ko.is_ko(r)) {
            return r;
        }
        return super.wait_ready(deadline);
    }

    public ko wait_ready(int seconds_deadline) {
        return wait_ready(new Date(System.currentTimeMillis() + seconds_deadline * 1000L));
    }

    public void stop() {
        if (peer != null) {
            if (!peer.is_finished()) {
                peer.disconnect(new seq_t(0), new reason_t("rpc_api service stopped"));
            }
        }
        handler.stop();
        super.stop();
        handler.signal_connected();

        rendezvous.flush();
    }

    public void join() {
        handler.join();
        super.join();
        peer = null;
    }

    @Override public void on_stop() {
        //if one stops stop the other
        if (super.is_active()) super.stop();
        if (handler.is_active()) handler.stop();
    }

    @Override public void attach(client peer_) {
        assert peer == null;
        peer = (peer_t) peer_;
    }

    @Override public void detach(client peer_) {
        assert peer_ == peer;
        peer.set_finish(); //follows client::on_destroy_ called from server thread - update clients - before going to gc
        if (peer.sock != null) {
            try {
                peer.sock.close();
            }
            catch (IOException e) {
            }
        }
        on_destroy_(peer);
        if (handler.stop_on_disconnection) {
            stop();
        }
    }

    public peer_t peer = null;

    public handler_daemon_t handler;

}

