package us.wallet.engine;
import java.util.ArrayList;                                                                    // ArrayList
import us.gov.crypto.base58;                                                                   // base58
import us.gov.io.blob_reader_t;                                                                // blob_reader_t
import us.gov.io.types.blob_t;                                                                 // blob_t
import us.gov.io.blob_writer_t;                                                                // blob_writer_t
import us.CFG;                                                                                 // CFG
import java.util.Date;                                                                         // Date
import us.gov.crypto.ec;                                                                       // ec
import static us.gov.crypto.types.*;                                                           // *
import static us.gov.id.types.*;                                                               // *
import static us.gov.socket.types.*;                                                           // *
import us.stdint.*;                                                                            // *
import static us.ko.is_ko;                                                                     // is_ko
import java.security.KeyPair;                                                                  // KeyPair
import us.ko;                                                                                  // ko
import static us.ko.ok;                                                                        // ok
import java.io.PrintStream;                                                                    // PrintStream
import static us.gov.io.types.blob_t.serid_t;                                                  // serid_t
import us.string;                                                                              // string

public class wallet_connection_t implements us.gov.io.seriable {


    public wallet_connection_t() {
        name_ = new string("default wallet");
        ssid = new string("");
        addr = new string("");
        subhome = new string("");
        ip4_endpoint = new ip4_endpoint_t();
        ts = new uint64_t(0);
    }

    public wallet_connection_t(String nm, String subhome_, ip4_endpoint_t ep) {
        name_ = new string(nm);
        ssid = new string("");
        addr = new string("");
        subhome = new string(subhome_);
        ip4_endpoint = new ip4_endpoint_t(ep);
        ts = new uint64_t(0);
    }

    public wallet_connection_t(uint64_t ts_, string addr_, string subhome_, string nm, string ssid_, ip4_endpoint_t ep) {
        name_ = new string(nm.value);
        ssid = new string(ssid_.value);
        addr = new string(addr_.value);
        subhome = new string(subhome_.value);
        ip4_endpoint = new ip4_endpoint_t(ep);
        ts = new uint64_t(ts_.value);
    }

    private wallet_connection_t(wallet_connection_t other) {
        name_ = new string(other.name_);
        ssid = new string(other.ssid);
        addr = new string(other.addr);
        subhome = new string(other.subhome);
        ip4_endpoint = new ip4_endpoint_t(other.ip4_endpoint);
        ts = new uint64_t(other.ts);
    }

    public wallet_connection_t copy() {
        wallet_connection_t wc = new wallet_connection_t(this);
        wc.name_.value = "copy_" + name_.value;
        wc.ssid.value = "";
        wc.addr.value = "";
        if (addr.value.startsWith("new")) {
            wc.subhome.value = subhome.value;
        }
        else {
            wc.subhome.value = "";
        }
        wc.ts.value = 0;
        return wc;
    }

    void write(blob_t blob) {
        serid_t serid = serial_id();
        int sz = (serid.value != 0 ? blob_writer_t.header_size() : 0) + blob_size();
        if (sz == 0) {
            blob.clear();
            return;
        }
        blob_writer_t w = new blob_writer_t(blob, sz);
        if (serid.value != 0) {
            w.write_header(serid);
        }
        to_blob(w);
        assert w.cur == blob.value.length;
    }

    public void dump(PrintStream os) {
        os.println("name " + name_.value);
        os.println("ssid " + ssid.value);
        os.println("addr " + addr.value);
        os.println("subhome " + subhome.value);
        os.println(ip4_endpoint.to_string());
    }

    public ko set_endpoint(ip4_endpoint_t ip4_endpoint_) {
        ip4_endpoint = ip4_endpoint_;
        return ok;
    }

    public String get_title() {
        String s = name_.value;
/*
        if (ssid.value != null && !ssid.value.isEmpty()) {
            s = s + "_" + ssid.value;
        }
        if (addr.value != null && !addr.value.isEmpty()) {
            s = s + "_" + addr.value;
        }
*/
        return s;
    }

    public boolean has_addr() {
        if (addr.value != null && !addr.value.isEmpty()) {
            return true;
        }
        return false;
    }

    public boolean is_custodial() {
        if (subhome.value != null && !subhome.value.isEmpty()) {
            return true;
        }
        return false;
    }

    public String get_address() {
        return "" + addr.value;
    }

    public String get_ssid() {
        return "" + ssid.value;
    }

    public String get_subtitle() {
        String s = "";
        if (ssid.value != null && !ssid.value.isEmpty()) {
            s = s + "_" + ssid.value;
        }
        if (addr.value != null && !addr.value.isEmpty()) {
            s = s + "_" + addr.value;
        }
        return s;
    }

    public String get_subhome() {
        if (is_custodial()) {
            return subhome.value;
        }
        return "";
    }

    @Override public serid_t serial_id() { return serid_t.no_header; }

    @Override public int blob_size() {
        int sz = blob_writer_t.blob_size(name_) +
                 blob_writer_t.blob_size(ssid) +
                 blob_writer_t.blob_size(addr) +
                 blob_writer_t.blob_size(subhome) +
                 blob_writer_t.blob_size(ip4_endpoint) +
                 blob_writer_t.blob_size(ts);
        return sz;
    }

    @Override public void to_blob(blob_writer_t writer) {
        writer.write(name_);
        writer.write(ssid);
        writer.write(addr);
        writer.write(subhome);
        writer.write(ip4_endpoint);
        writer.write(ts);
    }

    @Override public ko from_blob(blob_reader_t reader) {
        {
            ko r = reader.read(name_);
            if (ko.is_ko(r)) return r;
        }
        {
            ko r = reader.read(ssid);
            if (ko.is_ko(r)) return r;
        }
        {
            ko r = reader.read(addr);
            if (ko.is_ko(r)) return r;
        }
/*
        if (reader.header.serid.value == 'X' && reader.header.version.value == 11) {
            System.err.println("Remove old code");
            return new ko("KO 59200 Remove old code");
        }
        boolean miss_subhome = (reader.header.serid.value == 'X' && reader.header.version.value == 9);
        if (!miss_subhome) {
*/
        {
            ko r = reader.read(subhome);
            if (is_ko(r)) return r;
        }
        {
            ko r = reader.read(ip4_endpoint);
            if (ko.is_ko(r)) return r;
        }
        {
            ko r = reader.read(ts);
            if (ko.is_ko(r)) return r;
        }
        return ok;
    }

    public string name_;
    public string ssid;
    public string addr;
    public string subhome;
    public ip4_endpoint_t ip4_endpoint;
    public uint64_t ts = new uint64_t(0);
}

