#include "initiator_t.h"
#include "../trader_t.h"
#include "../traders_t.h"
#include <us/wallet/wallet/local_api.h>

#define loglevel "wallet/trader"
#define logclass "initiator_t"
#include <us/gov/logs.inc>

using namespace std;
using namespace us::wallet;
using namespace us::wallet::trader::bootstrap;
using c = us::wallet::trader::bootstrap::initiator_t;
using us::ko;

c::initiator_t(inverted_qr_t&& inverted_qr, wallet::local_api& w): inverted_qr(move(inverted_qr)), w(w) {
    log("constructor", "TRACE 8c", "qr:", this, inverted_qr.to_string());
}

c::~initiator_t() {
    log("destructor");
}

c::hash_t c::make_new_id() const { //TODO add some entropy. tid can be deduced with three inputs who-who-when
    us::gov::crypto::ripemd160 h;
    w.local_endpoint.hash(h);
    inverted_qr.endpoint.hash(h);
    h.write(trader->ts_creation);
    hash_t tid;
    h.finalize(tid); //unique hash for trade-id (tid)
    log("computed new id:", tid);
    return tid;
}

pair<ko, c::hash_t> c::start(trader_t& tder) {
    auto r = b::start(tder);
    log("start", "TRACE 8c");
    if (is_ko(r.first)) {
        return r;
    }
    log("start", "protocol selection", this, inverted_qr.protocol_selection.to_string2());
    trade_id = make_new_id();
    log("constructor initiator", trade_id);
    trader->init(trade_id, inverted_qr.endpoint, w);
    trader->set_state(conman::state_req_online);
    return make_pair(ok, trade_id);
}

void c::online(peer_t& peer) {
    b::online(peer);
    log("TRACE 8c");
    log("online", trader->has_protocol(), "protocol selection", this, inverted_qr.protocol_selection.to_string2());
    if (inverted_qr.protocol_selection.is_set()) {
        log("spawning dialogue_c");
        dialogue_c.initiate(peer, inverted_qr.endpoint.wloc, inverted_qr.protocol_selection);
    }
    else {
        log("spawning dialogue_a");
        dialogue_a.initiate(peer, inverted_qr.endpoint.wloc);
    }
}

ko c::on_a(int n) {
    log("on_a", n);
    if (n == 1) {
        log(dialogue_a_t::KO_43342);
        return dialogue_a_t::KO_43342;
    }
    return ok;
}

ko c::on_c(int n) {
    if (n == 1) {
        log(dialogue_c_t::KO_43344);
        return dialogue_c_t::KO_43344;
    }
    assert(n == 2 || n == 3 || n == 4);
    return ok;
}

c::inverted_qr_t::inverted_qr_t(qr_t&& qr): qr_t(move(qr)) {
}

