#include "protocols_t.h"
#include <functional>
#include <us/wallet/trader/types.h>

#define loglevel "wallet/trader/bootstrap"
#define logclass "protocols_t"
#include <us/gov/logs.inc>

using namespace us::wallet::trader;
using c = us::wallet::trader::bootstrap::protocols_t;

void c::dump(ostream& os) const {
    for (auto& i: *this) {
        os << i.first << ' ' << i.second << '\n';
    }
}

c c::filter_common(protocols_t a, protocols_t b) {
    function f = [](const item&a, const item&b)->bool {
        if (a.first != b.first) return a.first < b.first;
        return a.second < b.second;
    };
    protocols_t ans;
    if (a.empty() || b.empty()) return ans;
    sort(a.begin(), a.end(), f);
    sort(b.begin(), b.end(), f);
    protocols_t::const_iterator i = a.cbegin();
    protocols_t::const_iterator j = b.cbegin();
    while(true) {
        if (i == a.end() || i == b.end() || j == a.end() || j == b.end()) break;
        if (i->first == j->first && i->second == j->second) {
            ans.emplace_back(*i);
            ++i;
            ++j;
            continue;
        }
        if (!f(*i, *j)) std::swap(i, j);
        while(f(*i, *j)) {
            ++i;
            if (i == a.end() || i == b.end()) break;
        }
    }
    return ans;
}

void c::uniq() {
    set<protocol_selection_t> x;
    for (auto &i: *this) {
        x.emplace(i);        
    }
    clear();
    reserve(x.size());
    for (auto& i: x) {
        emplace_back(i);
    }
}

