#pragma once
#include <us/gov/io/seriable.h>
#include <us/gov/cash/locking_program_input_t.h>

#include "section_t.h"
#include "inputs_t.h"
#include "outputs_t.h"

namespace us::gov::cash::tx {

    struct section_t final: io::seriable {

        section_t();
        section_t(const hash_t& token);
        section_t(const section_t&);

    public:
        void add_input(const hash_t& addr, const cash_t& amount);
        void add_input(const hash_t& addr, const cash_t& amount, const locking_program_input_t&);
        void add_output(const hash_t& addr, const cash_t& amount);
        void write_sigmsg(sigmsg_hasher_t&, const sigcode_section_t&) const;
        void write_pretty(const string& prefix, ostream&) const;
        cash_t required_input() const;
        cash_t required_output() const;
        bool check_amounts() const;
        inline uint64_t uniq() const { return inputs.uniq(); }

    public:
        size_t blob_size() const override;
        void to_blob(blob_writer_t&) const override;
        ko from_blob(blob_reader_t&) override;

    public:
        hash_t token;
        inputs_t inputs;
        outputs_t outputs;
    };

}

