#pragma once
#include <unordered_map>
#include <mutex>
#include <string>

#include <us/gov/config.h>

#include "types.h"
#include "data_sources_index_t.h"

namespace us::wallet::engine {

    struct daemon_t;
    struct peer_t;


    struct data_source_t  {
        data_source_t() {
            address.zero();
        }
        data_source_t(const hash_t& address): address(address) {}
        virtual ~data_source_t() {}
        virtual void dump(const string& pfx, ostream&) const;

        void connect(const hash_t& addr) {
            address = addr;
        }
        hash_t address;
    };


    struct human_data_source_t: data_source_t  {
    };

    struct health_watch_data_source_t: data_source_t  {
    };

    struct data_sources_t final: unordered_map<string, data_source_t*> {
        using datagram = us::gov::socket::datagram;

    public:
        data_sources_t(daemon_t&);
        ~data_sources_t();

    public:
        data_source_t* get_data_source(const string& name); //returns soft pointer

    public:
        void dump(ostream&) const;


        ko get_index(data_sources_index_t& ans);
        ko connect(const string& name, const hash_t& address);

    public:
        daemon_t& daemon; //daemon mode, trading enabled

        #if CFG_LOGS == 1
            string logdir;
        #endif

        data_source_t* human{nullptr};

    private:
        mutable mutex mx;
    };

}

