#pragma once
#include <map>
#include <string>
#include <sstream>
#include <iostream>
#include <mutex>
#include <vector>

#include <us/gov/ko.h>
#include <us/gov/io/seriable_map.h>
#include <us/gov/io/seriable_vector.h>
#include <us/gov/io/seriable_map.h>

namespace us::wallet::trader {
    using namespace std;

    struct trader_t;

    struct data_t final: map<string, string> {
        using b = map<string, string>;

        struct diff_line {
            char action;
            string value;
        };

        struct diff_t: map<string, diff_line> {
            void filter(const string&);
            void dump(const string& pfx, ostream&) const;
            int seq{0};
        };

        data_t();
        data_t(const data_t&);
        data_t(data_t&&);
        ~data_t();

        data_t& operator = (const data_t&);

        bool operator == (const data_t&) const;
        bool operator != (const data_t&) const;

        void dump(const string& pfx, ostream&) const;
        void dump_(const string& pfx, ostream&) const;

        diff_t get_diff(const data_t& other) const;
        ko apply(const diff_t&);

        template<typename t>
        void add(const string& key, const t& value) {
            ostringstream os;
            os << value;
            emplace(key, value);
        }

    public:
        ko from(const string&);
        ko from(istream&);
        mutable mutex mx;
    };

    template<> void data_t::add(const string& key, const string& value);

}

