#pragma once
#include <string>
#include <vector>
#include <us/gov/ko.h>
#include <us/gov/io/seriable.h>

namespace us::wallet::trader {

    using namespace us::gov;

    struct protocol_selection_t: pair<string, string>, gov::io::seriable {
        using b = pair<string, string>;

        protocol_selection_t();
        protocol_selection_t(string&&, string&&);
        protocol_selection_t(const string&, const string&);
        protocol_selection_t(protocol_selection_t&&);
        protocol_selection_t(const protocol_selection_t&);

        protocol_selection_t& operator = (const protocol_selection_t&);
        inline bool operator == (const protocol_selection_t& other) const { return first == other.first && second == other.second; }
        inline bool operator != (const protocol_selection_t& other) const { return first != other.first || second != other.second; }
        bool operator < (const protocol_selection_t& other) const;

        void dump(const string& prefix, ostream&) const;
        bool is_set() const { return !first.empty() && !second.empty(); }

        void api_list_protocols(ostream&) const;

    public: //serialization blob
        size_t blob_size() const override;
        void to_blob(blob_writer_t&) const override;
        ko from_blob(blob_reader_t&) override;

    public:
        static protocol_selection_t from_string2(const string&);
        string to_string2() const;

    public: //serialization string
        string to_string() const;
        static void to_streamX(const string&, ostream&);
        static ko from_streamX(istream&, string&);
        void to_streamX(ostream&) const;
        ko from_streamX(istream&);

    };

    ostream& operator << (ostream&, const protocol_selection_t&);
}


