#pragma once
#include <tuple>

#include <us/gov/io/seriable.h>
#include <us/wallet/trader/params_t.h>
#include <us/wallet/trader/cert/doc0_t.h>

#include "workflow_t.h"

namespace us::wallet::trader::workflow {

    using namespace std;

    struct business_t;

    struct workflows_t final: vector<workflow_t*>, public us::gov::io::seriable {
        using b = vector<workflow_t*>;
        using doctypes_t = workflow_t::doctypes_t;
        using peer_t = workflow_t::peer_t;
        using ch_t = trader::ch_t;
        using doc0_t = us::wallet::trader::cert::doc0_t;

    public:
        workflows_t(business_t&);
        ~workflows_t() override;

    public:
        pair<workflow_t*, item_t*> find(const string& name) const;
        tuple<workflow_t*, item_t*, doc0_t*> read_item(const blob_t&) const;
        bool requires_online(const string& cmd) const;
        void help_online(const string& indent, ostream&) const;
        void help_onoffline(const string& indent, ostream&) const;
        void help_show(const string& indent, ostream&) const;
        void sig_reload(ostream&);
        void add(workflow_t*, ch_t&);
        bool sig_reset(ostream&);
        bool sig_hard_reset(ostream&);
        void on_file_updated(const string& path, const string& name, ch_t&);
        void recompute_doctypes(ch_t&);
        ko exec_offline(trader_t& tder, const string& cmd, ch_t&);
        ko exec_online(trader_t&, peer_t&, const string& cmd, ch_t&);
        ko rehome(const string& home, ch_t&);
        void doctypes(doctypes_t&) const; //consumer, producer
        ko update_item(const blob_t&, ch_t&);

    public:
        size_t blob_size() const override;
        void to_blob(blob_writer_t&) const override;
        ko from_blob(blob_reader_t&) override;

    public:
        using workflow_factories_t = workflow_t::factories_t;
        using workflow_factory_t = workflow_t::factory_t;
        using workflow_factory_id_t = workflow_t::factory_id_t;

        workflow_factories_t workflow_factories;

    public:
        string home;
        business_t& business;
    };

}

